// $Id: views_slideshow_ajax.js,v 1.1 2011/02/02 10:10:36 kentr Exp $

Drupal.behaviors.viewsSlideshowAjaxLoad = function (context) {
  var key;
  for (key in Drupal.settings.viewsSlideshowAjax) {
    var settings = Drupal.settings.viewsSlideshowAjax[key];
    
    // build array of loader functions to re-execute.  Use context param
    // to isolate to our slideshows.

    var view = $(settings['slideshow_view_dom_id']);
    
    /**
     * Store the settings ID with the slideshow object to facilitate later retrieval.
     */
    var slideshow = $(settings['slideshow_full_id'] );
    slideshow.data('settings_key', key);
    slideshow.ajaxComplete(function(e, request, options) {
          var response = Drupal.parseJson(request.responseText);
          if (typeof response.slideshow_count == 'number') {
            // debugger;
            // views_slideshow_ajax_log(response.slideshow_count);
            Drupal.settings.viewsSlideshowAjax[$(this).data('settings_key')].slideshow_count = response.slideshow_count;
          }
        }
    );
    /**
     * Add the extra containers to the items.
     * 
     * Assumes that the container for the slideshow element is the first
     * child of the main slideshow element.
     * 
     * @see views_slideshow.js
     */

    var $cont = $(settings['slideshow_full_id'] + ' :first');

    $cont.append('<div class=" views-slideshow-preload p0" style="display:none; position: absolute; left: -999999"></div>');
    $cont.append('<div class="views-slideshow-preload p1" style="display:none; position: absolute; left: -999999"></div>');
    delete $cont;
    
    // Initiate the load for the next slide
    
    // remove the "processed" class
    processedClass = settings['mode_name'] + '-processed';
    view.find('.' + processedClass).toggleClass(processedClass);
    delete view;

    /*
     * Add before / after callbacks to preload.
     */
    if (typeof Drupal.settings.viewsSlideshowSingleFrame[settings['slideshow_full_id']] != 'object') {
      views_slideshow_ajax_log("Settings not found for " + settings['slideshow_full_id']);
      continue;
    }

    var opts = Drupal.settings.viewsSlideshowSingleFrame[settings['slideshow_full_id']].opts;

    opts.beforeOrig = opts.before;
    opts.before = function(curr, next, opts) {

	      // targetId hasn't been set yet, so assume it's the first parent.
	      var meta = Drupal.settings.viewsSlideshowAjax['#' + $(opts.$cont).parent().attr('id')];
	      
	      // Gets called at very beginning of slideshow, so skip preload
	      if (meta.first_cycle) {
	        meta.first_cycle = 0;
	        return;
	      }
	
	      // Target of the next preload
	      var preloadTarget = (opts.currSlide + opts.slideCount - 1) % opts.slideCount;
	
	      var url = meta.ajax_url + ((meta.current_item + opts.slideCount - 1) % meta.slideshow_count);
	      
	      /**
	       * @todo: error checking before progressing.
	       */
	      views_slideshow_ajax_preload(opts.elements[preloadTarget], url);
	      
	      /*
	       * looping current_item to avoid problems with long running shows
	       * & out of range errors
	       */
	      meta.current_item = ((meta.current_item + 1) % meta.slideshow_count);
	
	      opts.beforeOrig(curr, next, opts);	
	      
	      delete result;
	      delete meta;
      };
 
    /* 
     * Now that we've added empty children, call the respective 
     * Drupal.behaviors method again to initiate the slideshow.
     */

    // Drupal.behaviors.viewsSlideshowSingleFrame(settings['slideshow_view_dom_id']);

    $(Drupal.settings.viewsSlideshowSingleFrame[settings['slideshow_full_id']].targetId).cycle(
      Drupal.settings.viewsSlideshowSingleFrame[settings['slideshow_full_id']].opts
    );
    /*
     * Populate the preload elements.
     * Done after initiating the slideshow to avoid delays.
     * 
     * We're assuming that we're starting at the beginning.
     * To start at the last slide viewed, change the '1' below as
     * needed to call the proper page. 
     * 
     * If synchronizing becomes a problem, this may need to be moved
     * above the slideshow init, or perhaps a config option added to
     * fire this before the init.
     */

    views_slideshow_ajax_preload($('.views-slideshow-preload', $cont)[0], settings['ajax_url'] + '1');
    
    delete opts;
    delete result;
  }
}

/**
 * Load a JSON result into a DOM preload element.
 *   
 * @param target
 *   A DOM object or jQuery selector into which the result should be
 *   loaded.  The inner HTML of the target will be replaced, leaving
 *   the target itself otherwise unmodified. 
 * @param url
 *   A string representing the full URL for the JSON query.
 *   The JSON response must be an object with a "display" property.
 */
function views_slideshow_ajax_preload(target, url) {
  var json = $.get(
    url,
    function(response) {    	
       var t = $(target);
       var result = Drupal.parseJson(response);
       result = $(result.display);
       t.html(result.html());

       /*
        * Could assume that the slide has the teaser has class '<mode>_hidden',
        * but may not be true...
        */

       t.attr({
         'class': result.attr('class'),
         'id': result.attr('id'),
         'style':  'display:none; position: absolute; left: -999999; ' + (result.attr('style') == null ? '' : result.attr('style'))
       });
    }
  );
}

function views_slideshow_ajax_log(){if(window.console&&window.console.log){window.console.log("[views slideshow ajax] "+Array.prototype.join.call(arguments," "));}}

/*members Drupal, $, $cont, ajax_url, append, attr, before, beforeOrig, behaviors, 
call, console, currSlide, current_item, cycle, display, elements, find, 
first_cycle, get, html, join, log, mode_name, opts, parent, parseJson, 
prototype, responseText, settings, slideCount, slideshow_count, 
slideshow_full_id, slideshow_view_dom_id, targetId, toggleClass, 
viewsSlideshowAjax, viewsSlideshowAjaxLoad, viewsSlideshowSingleFrame
*/